/**
 ** handler used to transparently handle adding child windows to tabs for both Android and iOS, on Android, the back button by default closes the tabGroup as that is
 ** main activity.
 **
 ** usage:
 ** var tabGroup = Titanium.UI.createTabGroup();
 **
 ** var tab1 = Titanium.UI.createTab({  
 **  		icon:'KS_nav_views.png',
 **		title:'Tab 1',
 **		window:win1
 **	});
 **
 ** var backHandler = new ingrs.UI.GenricTabHandler(tabGroup);
 ** backHandler.addWinToTabAndOpen(tab1,win2);
 ** ...add more windows as you like
 **
 ** 
 **  OR------
 **
 ** ingrs.UI.addWinToTabAndOpen(_tab,_win,_optionaltabGroup,_optionalprevWindow); //the optional parameters are only good if supplied on the first call
 **
 **/

/** private funciton to handle closing a close window **/
(function () {
	ingrs.UI.resetIfNotFlippable = function(_win) {
		if(typeof _win.onFlip === 'undefined'){
			ingrs.UI.resetTabWindow(_win);
		}
	}
	
	ingrs.UI.closeTabWindow = function(_tab,_tabGroup,_prevWindow,_closeAlreadyHandled) {
		if(typeof _tab === 'undefined' || _tab === null) {
			return;
		}
	
		/** nothing to close **/
		if(typeof _tab.cWindow === 'undefined' || _tab.cWindow === null) {
			return;
		}
		/** get the current window **/
		var currentW = _tab.cWindow;
		if(currentW.closedEvent !== true && currentW.noBackup === true) {
			return;
		}
		
		/** get its parent **/
		_tab.cWindow = _tab.cWindow.parentWindow;
		/** if there is a parent window, open it... **/
		if(typeof _tab.cWindow !== 'undefined') {
			_tab.cWindow.open({animate:true});
			ingrs.UI.resetIfNotFlippable(_tab.cWindow);
		}
		else {
            _tab.cWindow = _tab.window;
			if(typeof _prevWindow !== 'undefined' && _prevWindow !== null)  {
				_prevWindow.open({animate : true});
			}
			else if(typeof _tab.prevWindow !== 'undefined' && _tab.prevWindow !== null)  {
				_tab.prevWindow.open({animate : true});
			}
	
			/** ..else close the tabGroup **/
			if(typeof _tabGroup !== 'undefined' && _tabGroup !== null) {
				_tabGroup.close();
			}
			else if(typeof _tab.tabGroup !== 'undefined' && _tab.tabGroup !== null) {
				_tab.tabGroup.close();
			}
			/*else 
				_tab.mainWindow.close();*/
				
		}
		currentW.isClosed = true;
		/** close previously current window **/
		if(!_closeAlreadyHandled)
			currentW.close();
		
	}
	
	
	ingrs.UI.GenricTabHandler = function(_tabGroup,_prevWindow) {
		this.tabGroup = _tabGroup;
		this.prevWindow = _prevWindow;
		
		
		/** handle creation/addition a childwindow **/
		this.addWinToTabAndOpen = function(_tab,_win) {
			ingrs.UI.addWinToTabAndOpen(_tab,_win,this.tabGroup,this.prevWindow);
		}
	}
	ingrs.UI.closeWindowIfOpen = function(_win,_tab,_tabGroup,_prevWindow,_closeAlreadyHandled) {
		_win.closedEvent = true;
		if(!_win.isClosed)
			ingrs.UI.closeTabWindow(_tab,_tabGroup,_prevWindow,_closeAlreadyHandled)	
	}
	ingrs.UI.flippableTabWins = [];
	var prevOrientation = ingrs.UI.getOrientation(Titanium.Gesture.orientation);
	Ti.Gesture.addEventListener('orientationchange', function(e) {
		var currOrientation = ingrs.UI.getOrientation(Titanium.Gesture.orientation);
		if(typeof currOrientation !== 'undefined' && typeof prevOrientation !== 'undefined' && currOrientation !== prevOrientation && Titanium.Gesture.orientation !== 2){
			prevOrientation = currOrientation;
			ingrs.auth.blurFields();
		}
		var flipped = false;
		for(var i in ingrs.UI.flippableTabWins) {
			if(ingrs.UI.flippableTabWins[i].isActive === true && typeof Ti.API._activeTab !== 'undefined' && Ti.API._activeTab !== null && 
				(ingrs.UI.flippableTabWins[i].tabParent === Ti.API._activeTab || Ti.API._activeTab.window === ingrs.UI.flippableTabWins[i])) {
				if(flipped) {
					ingrs.log.error('Multiple Windows Actively Flipped');
				}
				ingrs.UI.flippableTabWins[i].onFlip();
				flipped = true;
			}
		}
	});
	ingrs.UI.addOrientationHandlers = function(_win) {
		var isIPhone = ingrs.isIphone();
		if(typeof _win.onFlip !== 'undefined'){
			_win.prevOrientation = ingrs.UI.getOrientation(Titanium.Gesture.orientation);
			_win.isActive = false;
			ingrs.UI.flippableTabWins.push(_win);
			if(typeof _win.flipHandlerAdded === 'undefined' || _win.flipHandlerAdded === false) {
				if(isIPhone) {
					_win.addEventListener('focus',function() {
						_win.isActive = true;
						_win.onFlip();
					});
					_win.addEventListener('blur', function(e) {
						_win.isActive = false;
						_win.prevOrientation = ingrs.UI.getOrientation(Titanium.Gesture.orientation);
					});
				}
				else {
					_win.addEventListener('open',function() {
						_win.isActive = true;
						_win.onFlip();
					});					
					_win.addEventListener('close', function(e) {
						_win.isActive = false;
						_win.prevOrientation = ingrs.UI.getOrientation(Titanium.Gesture.orientation);
					});
				}
				_win.flipHandlerAdded = true;
			}
		}
		else {
			ingrs.UI.resetTabWindow(_win);
		}
	};
	
	/** handle creation/addition a childwindow **/
	ingrs.UI.addWinToTabAndOpen = function(_tab,_win,_tabGroup,_prevWindow) {	
	
		if(_tab === null || typeof _tab === 'undefined' ||
		   _win === null || typeof _win === 'undefined') {
		   	return;
		}
		/** if it is iOS handle normally **/
		_win.tabParent = _tab;
		if(ingrs.isIphone()){
			ingrs.UI.addOrientationHandlers(_win);
			_tab.open(_win);
			if(typeof _tab.cWindow === 'undefined') {
				_tab.cWindow = _tab.window;
			}
			_win.parentWindow = _tab.cWindow;
			_tab.cWindow = _win;
			_win.closeMe = function() {
				_win.close();
			}
			return;
		}
		if(ingrs.isAndroid()){
			_win.tab = _tab;
			// /** attach the window to the tab **/
			var cWin;
			_win.addEventListener('android:back',function(e) {
				ingrs.UI.closeTabWindow(_tab,_tabGroup,_prevWindow);
			});
			if(typeof _tab.mainWindow === 'undefined') {
				/** first child window, setup the "android:back" handler on the mainWindow(one that came with the tab at the start) **/
				_tab.mainWindow = _tab.window;
				
				_tab.mainWindow.addEventListener('android:back',function(e) {
					ingrs.UI.closeTabWindow(e.source.tab,_tabGroup,_prevWindow);
				});
				/** set the current Window for tab - the 'main' window **/
				_tab.cWindow = _tab.window;
				/*Ti.App.addEventListener('closeFull', function(e) {
					if(e.source.navBarHidden === true) {
						ingrs.UI.closeWindowIfOpen(e.source,_tab,_tabGroup,_prevWindow,true)
					}
				});*/
			}
			else {
				/** not the first child, note down the current window **/
				cWin = _tab.cWindow;
			} 
			if(typeof _tab.cWindow !== 'undefined') {
				/** set the parent if one is available **/
				_win.parentWindow = _tab.cWindow;
			}

			ingrs.UI.addOrientationHandlers(_win);
			/** set the current window **/
			_tab.cWindow = _win;   // edit by Fede - commenting out this line then it works
			
			_tab.add(_win);
			_win.isClosed = false;
			_win.closedEvent = false;
			_win.open({animate:true});
			/** close the current window **/
			if(typeof cWin !== 'undefined' && cWin !== _tab.mainWindow) {
				// cWin.isClosed = true;
				// cWin.close();
			}
			_win.addEventListener('close', function(e) {
				ingrs.UI.closeWindowIfOpen(_win,_tab,_tabGroup,_prevWindow,false);
			});
			
		}
	}
	
	ingrs.UI.closeAllWindowsTillMain = function(_tab,_attributes) {
		ingrs.UI.closeAllWindowsTillMainLogic(_tab,true,_attributes);
	}
	
	ingrs.UI.closeAllWindowsTillMainLogic = function(_tab,_openMain,_attributes) {
	
		if(typeof _tab === 'undefined' || _tab === null) {
			return;
		}
	
		var objs = [];
		while(typeof _tab.cWindow !== 'undefined' && _tab.cWindow !== null && _tab.cWindow !== _tab.window) {
			/** get the current window **/
			var currentW = _tab.cWindow;
			
			if(typeof _attributes !== 'undefined') {
				/* do a match */
				var match = true;
				for(var i in _attributes){
					if(currentW[i] !== _attributes[i]) {
						match = false;
						break;
					}
				}
				if(match === true) {
					if(ingrs.isIphone()) {
						break;
					}
					else {
						currentW.open({animate:true});
						ingrs.UI.resetIfNotFlippable(currentW);
						return;
					}
				}
			}
			
			/** get its parent **/
			_tab.cWindow = _tab.cWindow.parentWindow;
			currentW.isClosed = true;
			/** close previously current window **/
			if(ingrs.isIphone()){
				objs.push(currentW);
			}
			else {
				currentW.close();
			}
		}
		if(ingrs.isIphone()){
			for(var i=objs.length -1;i>-1;i--){
				objs[i].closeMe();
			}
		}
		if(ingrs.isAndroid()){
            if(typeof _tab.cWindow !== 'undefined'){
            	if(_openMain === true) {
                	_tab.window.open();
                }
                ingrs.UI.resetIfNotFlippable(_tab.window);
            }
		}	
	}
	
	ingrs.UI.resetTabWindow = function(_win) {
		if (ingrs.isIphone()) {
			_win.showNavBar();
		}
		Ti.App.fireEvent('showTabGroup');
		if(typeof _win.innerContainer !== 'undefined') {
			_win.innerContainer.top = 0;
		}
	}
})();
